var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { ConsentContractError, EVMContractAddress, TokenId, RequestForData, BlockNumber, HexString, ConsentToken, DataPermissions, BigNumberString, } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { injectable } from "inversify";
import { ok, err, okAsync, ResultAsync } from "neverthrow";
import { ResultUtils } from "neverthrow-result-utils";
import { ERC7529Contract } from "../implementations/ERC7529Contract.js";
import { EConsentRoles, Tag, ContractsAbis, } from "../interfaces/index.js";
let ConsentContract = class ConsentContract extends ERC7529Contract {
    constructor(providerOrSigner, contractAddress, cryptoUtils) {
        super(providerOrSigner, contractAddress, ContractsAbis.ConsentAbi.abi);
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.cryptoUtils = cryptoUtils;
        this.filters = {
            Transfer: (fromAddress, toAddress) => {
                return this.contract.filters.Transfer(fromAddress, toAddress);
            },
            RequestForData: (ownerAddress) => {
                return this.contract.filters.RequestForData(ownerAddress);
            },
        };
    }
    getContractAddress() {
        return this.contractAddress;
    }
    optIn(tokenId, agreementFlags, overrides) {
        return this.writeToContract("optIn", [tokenId, agreementFlags], overrides);
    }
    // TODO: add data permissions param
    encodeOptIn(tokenId, agreementFlags) {
        return HexString(this.contract.interface.encodeFunctionData("optIn", [
            tokenId,
            agreementFlags,
        ]));
    }
    restrictedOptIn(tokenId, agreementFlags, signature, overrides) {
        return this.writeToContract("restrictedOptIn", [tokenId, agreementFlags, signature], overrides);
    }
    encodeRestrictedOptIn(tokenId, signature, agreementFlags) {
        return HexString(this.contract.interface.encodeFunctionData("restrictedOptIn", [
            tokenId,
            agreementFlags,
            signature,
        ]));
    }
    anonymousRestrictedOptIn(tokenId, agreementFlags, signature, overrides) {
        return this.writeToContract("anonymousRestrictedOptIn", [tokenId, agreementFlags, signature], overrides);
    }
    encodeAnonymousRestrictedOptIn(tokenId, signature, agreementFlags) {
        return HexString(this.contract.interface.encodeFunctionData("anonymousRestrictedOptIn", [
            tokenId,
            agreementFlags,
            signature,
        ]));
    }
    optOut(tokenId, overrides) {
        return this.writeToContract("optOut", [tokenId], overrides);
    }
    encodeOptOut(tokenId) {
        return HexString(this.contract.interface.encodeFunctionData("optOut", [tokenId]));
    }
    agreementFlags(tokenId) {
        return ResultAsync.fromPromise(this.contract.agreementFlagsArray(tokenId), (e) => {
            return this.generateError(e, "Unable to call agreementFlagsArray()");
        });
    }
    getMaxCapacity() {
        return ResultAsync.fromPromise(this.contract.maxCapacity(), (e) => {
            return this.generateError(e, "Unable to call maxCapacity()");
        }).map((bigCapacity) => {
            return Number(bigCapacity);
        });
    }
    updateMaxCapacity(maxCapacity, overrides) {
        return this.writeToContract("updateMaxCapacity", [maxCapacity], overrides);
    }
    updateAgreementFlags(tokenId, newAgreementFlags, overrides) {
        return this.writeToContract("updateAgreementFlags", [tokenId, newAgreementFlags], overrides);
    }
    encodeUpdateAgreementFlags(tokenId, agreementFlags) {
        return HexString(this.contract.interface.encodeFunctionData("updateAgreementFlags", [
            tokenId,
            agreementFlags,
        ]));
    }
    requestForData(ipfsCID, overrides) {
        return this.writeToContract("requestForData", [ipfsCID], overrides);
    }
    // Returns the address Consent contract contract's owner
    // Note that the address on index 0 of the DEFAULT_ADMIN_ROLE members is the contract owner
    getConsentOwner() {
        return ResultAsync.fromPromise(this.contract.getRoleMember(EConsentRoles.DEFAULT_ADMIN_ROLE, 0), (e) => {
            return this.generateError(e, "Unable to call getRoleMember()");
        });
    }
    // Returns an array of address that has the DEFAULT_ADMIN_ROLE on the Consent contract
    // Note that the address on index 0 is the contract owner
    getDefaultAdminRoleMembers() {
        return ResultAsync.fromPromise(this.contract.getRoleMemberCount(EConsentRoles.DEFAULT_ADMIN_ROLE), (e) => {
            return this.generateError(e, "Unable to call getDefaultAdminRoleMembers()");
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < Number(memberCount); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return ResultUtils.combine(memberIndexArray.map((index) => {
                return ResultAsync.fromPromise(this.contract.getRoleMember(EConsentRoles.DEFAULT_ADMIN_ROLE, index), (e) => {
                    return this.generateError(e, "Unable to call getRoleMember()");
                });
            }));
        });
    }
    getSignerRoleMembers() {
        return ResultAsync.fromPromise(this.contract.getRoleMemberCount(EConsentRoles.SIGNER_ROLE), (e) => {
            return this.generateError(e, "Unable to call getSignerRoleMembers()");
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < Number(memberCount); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return ResultUtils.combine(memberIndexArray.map((index) => {
                return ResultAsync.fromPromise(this.contract.getRoleMember(EConsentRoles.PAUSER_ROLE, index), (e) => {
                    return this.generateError(e, "Unable to call getRoleMember() for SIGNER_ROLE");
                });
            }));
        });
    }
    getPauserRoleMembers() {
        return ResultAsync.fromPromise(this.contract.getRoleMemberCount(EConsentRoles.PAUSER_ROLE), (e) => {
            return this.generateError(e, "Unable to call getPauserRoleMembers()");
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < Number(memberCount); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return ResultUtils.combine(memberIndexArray.map((index) => {
                return ResultAsync.fromPromise(this.contract.getRoleMember(EConsentRoles.PAUSER_ROLE, index), (e) => {
                    return this.generateError(e, "Unable to call getRoleMember() for PAUSER_ROLE");
                });
            }));
        });
    }
    getRequesterRoleMembers() {
        return ResultAsync.fromPromise(this.contract.getRoleMemberCount(EConsentRoles.REQUESTER_ROLE), (e) => {
            return this.generateError(e, "Unable to call getRequesterRoleMembers()");
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < Number(memberCount); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return ResultUtils.combine(memberIndexArray.map((index) => {
                return ResultAsync.fromPromise(this.contract.getRoleMember(EConsentRoles.REQUESTER_ROLE, index), (e) => {
                    return this.generateError(e, "Unable to call getRoleMember() for REQUESTER_ROLE");
                });
            }));
        });
    }
    balanceOf(address) {
        return ResultAsync.fromPromise(this.contract.balanceOf(address), (e) => {
            return this.generateError(e, "Unable to call balanceOf()");
        }).map((numberOfTokens) => {
            return Number(numberOfTokens);
        });
    }
    ownerOf(tokenId) {
        return ResultAsync.fromPromise(this.contract.ownerOf(tokenId), (e) => {
            return this.generateError(e, "Unable to call ownerOf()");
        });
    }
    tokenURI(tokenId) {
        return ResultAsync.fromPromise(this.contract.tokenURI(tokenId), (e) => {
            return this.generateError(e, "Unable to call tokenURI()");
        }).orElse((error) => {
            // The contract reverts with this message if tokenId does not exist
            if (error.reason === "ERC721: operator query for nonexistent token") {
                return ok(null);
            }
            return err(error);
        });
    }
    queryFilter(eventFilter, fromBlock, toBlock) {
        return ResultAsync.fromPromise(this.contract.queryFilter(eventFilter, fromBlock, toBlock), (e) => {
            return this.generateError(e, "Unable to call queryFilter()");
        });
    }
    getConsentToken(tokenId) {
        // Get the agreement flags of the user's current consent token
        return ResultUtils.combine([
            this.ownerOf(tokenId),
            this.agreementFlags(tokenId),
        ]).andThen(([ownerAddress, agreementFlags]) => {
            return okAsync(new ConsentToken(this.contractAddress, ownerAddress, tokenId, new DataPermissions(agreementFlags)));
        });
    }
    addDomain(domain, overrides) {
        return this.writeToContract("addDomain", [domain], overrides);
    }
    removeDomain(domain, overrides) {
        return this.writeToContract("removeDomain", [domain], overrides);
    }
    getDomains() {
        return ResultAsync.fromPromise(
        // returns array of domains
        this.contract.getDomains(), (e) => {
            return this.generateError(e, "Unable to call getDomains()");
        });
    }
    getRequestForDataListByRequesterAddress(requesterAddress, fromBlock, toBlock) {
        return this.queryFilter(this.filters.RequestForData(requesterAddress), fromBlock, toBlock).map((logsEvents) => {
            return logsEvents.reduce((acc, logEvent) => {
                if (logEvent instanceof ethers.EventLog) {
                    acc.push(new RequestForData(this.getContractAddress(), logEvent.args.requester, logEvent.args.ipfsCID, BlockNumber(logEvent.blockNumber)));
                }
                return acc;
            }, new Array());
        });
    }
    getLatestTokenIdByOptInAddress(optInAddress) {
        return this.queryFilter(this.filters.Transfer(null, optInAddress), undefined, undefined).map((logsEvents) => {
            if (logsEvents.length == 0) {
                return null;
            }
            const latestOptinEvent = logsEvents.reduce((latestEvent, logEvent) => logEvent.blockNumber > latestEvent.blockNumber
                ? logEvent
                : latestEvent, logsEvents[0]);
            if (latestOptinEvent instanceof ethers.EventLog) {
                if (latestOptinEvent.args.tokenId != null) {
                    return TokenId(latestOptinEvent.args.tokenId);
                }
            }
            return null;
        });
    }
    disableOpenOptIn(overrides) {
        return this.writeToContract("disableOpenOptIn", [], overrides);
    }
    enableOpenOptIn(overrides) {
        return this.writeToContract("enableOpenOptIn", [], overrides);
    }
    baseURI() {
        return ResultAsync.fromPromise(this.contract.baseURI(), (e) => {
            return this.generateError(e, "Unable to call baseURI()");
        });
    }
    setBaseURI(baseUri, overrides) {
        return this.writeToContract("setBaseURI", [baseUri], overrides);
    }
    hasRole(role, address) {
        return ResultAsync.fromPromise(this.contract.hasRole(EConsentRoles[role], address), (e) => {
            return this.generateError(e, "Unable to call hasRole()");
        });
    }
    grantRole(role, address, overrides) {
        return this.writeToContract("grantRole", [role, address], overrides);
    }
    revokeRole(role, address, overrides) {
        return this.writeToContract("revokeRole", [role, address], overrides);
    }
    renounceRole(role, address, overrides) {
        return this.writeToContract("renounceRole", [role, address], overrides);
    }
    getQueryHorizon() {
        return ResultAsync.fromPromise(this.contract.queryHorizon(), (e) => {
            return this.generateError(e, "Unable to call queryHorizon()");
        }).map((queryHorizon) => BlockNumber(Number(queryHorizon)));
    }
    setQueryHorizon(blockNumber, overrides) {
        return this.writeToContract("setQueryHorizon", [blockNumber], overrides);
    }
    estimateGasLimitForSetQueryHorizon(blockNumber, overrides) {
        return ResultAsync.fromPromise(this.contract.estimateGas["setQueryHorizon"](blockNumber, {
            overrides,
        }), (e) => this.generateError(e, `Failed to estimate gas with error: ${e}`)).map((bnGas) => {
            return BigNumberString(bnGas.toString());
        });
    }
    // Get the number of opted in addresses
    totalSupply() {
        return ResultAsync.fromPromise(this.contract.totalSupply(), (e) => {
            return this.generateError(e, "Unable to call totalSupply()");
        }).map((totalSupply) => Number(totalSupply));
    }
    openOptInDisabled() {
        return ResultAsync.fromPromise(this.contract.openOptInDisabled(), (e) => {
            return this.generateError(e, "Unable to call openOptInDisabled()");
        });
    }
    // Marketplace functions
    getMaxTags() {
        return ResultAsync.fromPromise(this.contract.maxTags(), (e) => {
            return this.generateError(e, "Unable to call maxTags()");
        }).map((num) => {
            return Number(num);
        });
    }
    getNumberOfStakedTags() {
        return ResultAsync.fromPromise(this.contract.getNumberOfStakedTags(), (e) => {
            return this.generateError(e, "Unable to call getNumberOfStakedTags()");
        }).map((num) => {
            return Number(num);
        });
    }
    getTagArray() {
        return ResultAsync.fromPromise(this.contract.getTagArray(), (e) => {
            return this.generateError(e, "Unable to call getTagArray()");
        }).map((tags) => {
            return tags.map((tag) => {
                return new Tag(tag.slot ? BigNumberString(tag.slot.toString()) : null, tag.tag, tag.staker);
            });
        });
    }
    newGlobalTag(tag, newStakeAmount, overrides) {
        return this.writeToContract("newGlobalTag", [tag, newStakeAmount], overrides);
    }
    newLocalTagUpstream(tag, newStakeAmount, existingStakeAmount, overrides) {
        return this.writeToContract("newLocalTagUpstream", [tag, newStakeAmount, existingStakeAmount], overrides);
    }
    newLocalTagDownstream(tag, existingStakeAmount, newStakeAmount, overrides) {
        return this.writeToContract("newLocalTagDownstream", [tag, existingStakeAmount, newStakeAmount], overrides);
    }
    replaceExpiredListing(tag, stakeAmount, overrides) {
        return this.writeToContract("replaceExpiredListing", [tag, stakeAmount], overrides);
    }
    removeListing(tag, overrides) {
        return this.writeToContract("removeListing", [tag], overrides);
    }
    getSignature(values) {
        const types = ["address", "uint256"];
        return this.cryptoUtils.getSignature(this.providerOrSigner, types, values);
    }
    generateContractSpecificError(msg, e, transaction) {
        return new ConsentContractError(msg, e, transaction);
    }
};
ConsentContract = __decorate([
    injectable(),
    __metadata("design:paramtypes", [Object, String, Object])
], ConsentContract);
export { ConsentContract };
//# sourceMappingURL=ConsentContract.js.map