var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { EVMContractAddress, TokenId, CrumbsContractError, HexString, } from "@snickerdoodlelabs/objects";
import { injectable } from "inversify";
import { okAsync, errAsync, ResultAsync } from "neverthrow";
import { BaseContract } from "../implementations/BaseContract.js";
import { ContractsAbis, } from "../interfaces/objects/index.js";
let CrumbsContract = class CrumbsContract extends BaseContract {
    constructor(providerOrSigner, contractAddress) {
        super(providerOrSigner, contractAddress, ContractsAbis.CrumbsAbi.abi);
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
    }
    getContractAddress() {
        return this.contractAddress;
    }
    addressToCrumbId(accountAddress) {
        return ResultAsync.fromPromise(this.contract.addressToCrumbId(accountAddress), (e) => {
            return this.generateError(e, "Unable to call addressToCrumbId()");
        }).map((tokenId) => {
            // The contract returns 0 for an address that does not have a Crumb Id
            // Handle by returning null
            if (tokenId == 0n) {
                return null;
            }
            return TokenId(tokenId);
        });
    }
    tokenURI(tokenId) {
        return ResultAsync.fromPromise(this.contract.tokenURI(tokenId), (e) => {
            return this.generateError(e, "Unable to call tokenURI()");
        }).orElse((error) => {
            // The contract reverts with this message if tokenId does not exist
            if (error.reason === "ERC721: operator query for nonexistent token") {
                return okAsync(null);
            }
            return errAsync(error);
        });
    }
    createCrumb(crumbId, tokenUri, overrides) {
        return this.writeToContract("createCrumb", [crumbId, tokenUri], overrides);
    }
    encodeCreateCrumb(crumbId, crumbContent) {
        return HexString(this.contract.interface.encodeFunctionData("createCrumb", [
            crumbId,
            crumbContent,
        ]));
    }
    burnCrumb(crumbId, overrides) {
        return this.writeToContract("createCrumb", [crumbId], overrides);
    }
    encodeBurnCrumb(crumbId) {
        return HexString(this.contract.interface.encodeFunctionData("burnCrumb", [crumbId]));
    }
    updateTokenURI(crumbId, tokenURI, overrides) {
        return this.writeToContract("updateTokenURI", [crumbId, tokenURI], overrides);
    }
    generateContractSpecificError(msg, e, transaction) {
        return new CrumbsContractError(msg, e, transaction);
    }
};
CrumbsContract = __decorate([
    injectable(),
    __metadata("design:paramtypes", [Object, String])
], CrumbsContract);
export { CrumbsContract };
//# sourceMappingURL=CrumbsContract.js.map