import { EVMAccountAddress, EVMContractAddress, TokenUri, TokenId, ERC1155ContractError, BlockchainCommonErrors, DomainName, TokenAmount } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { ResultAsync } from "neverthrow";
import { BaseContract } from "../implementations/BaseContract.js";
import { IEthersContractError } from "../implementations/BlockchainErrorMapper.js";
import { ERewardRoles } from "../interfaces/enums/ERewardRoles.js";
import { ContractOverrides, IERC1155RewardContract, WrappedTransactionResponse } from "../interfaces/index.js";
export declare class ERC1155RewardContract extends BaseContract<ERC1155ContractError> implements IERC1155RewardContract {
    protected providerOrSigner: ethers.Provider | ethers.Signer;
    protected contractAddress: EVMContractAddress;
    constructor(providerOrSigner: ethers.Provider | ethers.Signer, contractAddress: EVMContractAddress);
    getContractAddress(): EVMContractAddress;
    getOwner(): ResultAsync<EVMAccountAddress, ERC1155ContractError | BlockchainCommonErrors>;
    getDefaultAdminRoleMembers(): ResultAsync<EVMAccountAddress[], ERC1155ContractError | BlockchainCommonErrors>;
    getMinterRoleMembers(): ResultAsync<EVMAccountAddress[], ERC1155ContractError | BlockchainCommonErrors>;
    tokenURI(tokenId: TokenId): ResultAsync<TokenUri, ERC1155ContractError | BlockchainCommonErrors>;
    setTokenURI(tokenId: TokenId, newURI: TokenUri, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    balanceOf(address: EVMAccountAddress, tokenId: TokenId): ResultAsync<number, ERC1155ContractError | BlockchainCommonErrors>;
    balanceOfBatch(addresses: EVMAccountAddress[], tokenIds: TokenId[]): ResultAsync<number[], ERC1155ContractError | BlockchainCommonErrors>;
    hasRole(role: keyof typeof ERewardRoles, address: EVMAccountAddress): ResultAsync<boolean, ERC1155ContractError | BlockchainCommonErrors>;
    grantRole(role: keyof typeof ERewardRoles, address: EVMAccountAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    revokeRole(role: keyof typeof ERewardRoles, address: EVMAccountAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    renounceRole(role: keyof typeof ERewardRoles, address: EVMAccountAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    addDomain(domain: DomainName, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    removeDomain(domain: DomainName, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    getDomains(): ResultAsync<DomainName[], BlockchainCommonErrors | ERC1155ContractError>;
    isApprovedForAll(tokenOwnerAddress: EVMAccountAddress, operatorToApprove: EVMAccountAddress): ResultAsync<boolean, ERC1155ContractError | BlockchainCommonErrors>;
    setApproveForAll(addressToApprove: EVMAccountAddress, approved: boolean, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    safeTransferFrom(from: EVMAccountAddress, to: EVMAccountAddress, tokenId: TokenId, amount: TokenAmount, data: string, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    safeBatchTransferFrom(from: EVMAccountAddress, to: EVMAccountAddress, tokenIds: TokenId[], amounts: TokenAmount[], data: string, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    totalSupply(tokenId: TokenId): ResultAsync<bigint, ERC1155ContractError | BlockchainCommonErrors>;
    exists(tokenId: TokenId): ResultAsync<boolean, ERC1155ContractError | BlockchainCommonErrors>;
    rewardsTokenCount(): ResultAsync<bigint, ERC1155ContractError | BlockchainCommonErrors>;
    createNewTokens(newURIs: TokenUri[], overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC1155ContractError>;
    protected generateContractSpecificError(msg: string, e: IEthersContractError, transaction: ethers.Transaction | null): ERC1155ContractError;
    filters: {
        TransferSingle: (operatorAddress: EVMAccountAddress | null, fromAddress: EVMAccountAddress | null, toAddress: EVMAccountAddress | null, tokenId: TokenId | null, value: bigint) => ethers.DeferredTopicFilter;
    };
}
//# sourceMappingURL=ERC1155RewardContract.d.ts.map