import { BaseURI, RewardsFactoryError, ECreatedRewardType, BlockchainCommonErrors } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { ResultAsync } from "neverthrow";
import { BaseContract } from "../../implementations/BaseContract.js";
import { IEthersContractError } from "../../implementations/BlockchainErrorMapper.js";
import { ContractOverrides, IRewardsContractFactory } from "../../interfaces/index.js";
import { WrappedTransactionResponse } from "../../interfaces/objects/index.js";
export declare class RewardsContractFactory extends BaseContract<RewardsFactoryError> implements IRewardsContractFactory {
    protected providerOrSigner: ethers.Provider | ethers.Signer;
    protected rewardType: ECreatedRewardType;
    protected erc721ContractFactory: ethers.ContractFactory;
    protected erc20ContractFactory: ethers.ContractFactory;
    protected rewardTypeToDeploy: ECreatedRewardType;
    constructor(providerOrSigner: ethers.Provider | ethers.Signer, rewardType: ECreatedRewardType);
    deployERC721Reward(name: string, symbol: string, baseURI: BaseURI, overrides: ContractOverrides, omitGasFee?: boolean): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | RewardsFactoryError>;
    estimateGasToDeployERC721Contract(name: string, symbol: string, baseURI: BaseURI): ResultAsync<bigint, RewardsFactoryError | BlockchainCommonErrors>;
    deployERC20Reward(name: string, symbol: string, overrides: ContractOverrides, omitGasFee?: boolean): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | RewardsFactoryError>;
    estimateGasToDeployERC20Contract(name: string, symbol: string): ResultAsync<bigint, RewardsFactoryError | BlockchainCommonErrors>;
    protected generateContractSpecificError(msg: string, e: IEthersContractError, transaction: ethers.Transaction | null): RewardsFactoryError;
    protected writeToContractFactory(functionName: string, functionParams: any[], rewardType: ECreatedRewardType, overrides?: ContractOverrides, isDeployingContract?: boolean): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | RewardsFactoryError>;
}
//# sourceMappingURL=RewardsContractFactory.d.ts.map