import { BaseURI, RewardsFactoryError, ECreatedRewardType, BlockchainCommonErrors } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { ResultAsync } from "neverthrow";
import { Provider, ContractFactory, Wallet } from "zksync-ethers";
import { BaseContract } from "../../implementations/BaseContract.js";
import { IEthersContractError } from "../../implementations/BlockchainErrorMapper.js";
import { ContractOverrides, IRewardsContractFactory } from "../../interfaces/index.js";
import { WrappedTransactionResponse } from "../../interfaces/objects/index.js";
export declare class ZkSyncRewardsContractFactory extends BaseContract<RewardsFactoryError> implements IRewardsContractFactory {
    protected providerOrSigner: Provider | Wallet;
    protected rewardType: ECreatedRewardType;
    protected contractFactory: ContractFactory;
    protected rewardTypeToDeploy: ECreatedRewardType;
    constructor(providerOrSigner: Provider | Wallet, // needs to use ZkSync provider and wallet
    rewardType: ECreatedRewardType);
    deployERC721Reward(name: string, symbol: string, baseURI: BaseURI, overrides: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | RewardsFactoryError>;
    estimateGasToDeployERC721Contract(name: string, symbol: string, baseURI: BaseURI): ResultAsync<bigint, RewardsFactoryError | BlockchainCommonErrors>;
    deployERC20Reward(name: string, symbol: string, overrides: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | RewardsFactoryError>;
    estimateGasToDeployERC20Contract(name: string, symbol: string): ResultAsync<bigint, RewardsFactoryError | BlockchainCommonErrors>;
    protected generateContractSpecificError(msg: string, e: IEthersContractError, transaction: ethers.Transaction | null): RewardsFactoryError;
    protected writeToContractFactory(functionName: string, functionParams: any[], overrides?: ContractOverrides, isDeployingContract?: boolean): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | RewardsFactoryError>;
}
//# sourceMappingURL=ZkSyncRewardsContractFactory.d.ts.map